"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class CheckBackupRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SILENT_FIELD_NUMBER: builtins.int
    silent: builtins.bool
    def __init__(self,
        *,
        silent: builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["silent",b"silent"]) -> None: ...
global___CheckBackupRequest = CheckBackupRequest

class CheckBackupResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ID_FIELD_NUMBER: builtins.int
    id: typing.Text
    def __init__(self,
        *,
        id: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["id",b"id"]) -> None: ...
global___CheckBackupResponse = CheckBackupResponse

class CreateBackupRequest(google.protobuf.message.Message):
    """Timestamp must be in UTC"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TIMESTAMP_FIELD_NUMBER: builtins.int
    TIMEZONE_OFFSET_FIELD_NUMBER: builtins.int
    timestamp: builtins.int
    timezone_offset: builtins.int
    def __init__(self,
        *,
        timestamp: builtins.int = ...,
        timezone_offset: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["timestamp",b"timestamp","timezone_offset",b"timezone_offset"]) -> None: ...
global___CreateBackupRequest = CreateBackupRequest

class ListBackupsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___ListBackupsRequest = ListBackupsRequest

class BackupInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ID_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    id: typing.Text
    timestamp: builtins.int
    name: typing.Text
    """uint32 timezone_offset = 3;"""

    def __init__(self,
        *,
        id: typing.Text = ...,
        timestamp: builtins.int = ...,
        name: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["id",b"id","name",b"name","timestamp",b"timestamp"]) -> None: ...
global___BackupInfo = BackupInfo

class ListBackupsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    INFO_FIELD_NUMBER: builtins.int
    @property
    def info(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___BackupInfo]: ...
    def __init__(self,
        *,
        info: typing.Optional[typing.Iterable[global___BackupInfo]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["info",b"info"]) -> None: ...
global___ListBackupsResponse = ListBackupsResponse

class RestoreBackupRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ID_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    TIMEZONE_OFFSET_FIELD_NUMBER: builtins.int
    id: typing.Text
    timestamp: builtins.int
    timezone_offset: builtins.int
    def __init__(self,
        *,
        id: typing.Text = ...,
        timestamp: builtins.int = ...,
        timezone_offset: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["id",b"id","timestamp",b"timestamp","timezone_offset",b"timezone_offset"]) -> None: ...
global___RestoreBackupRequest = RestoreBackupRequest
