# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: antiklepto.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10\x61ntiklepto.proto\x12\x14shiftcrypto.bitbox02\"3\n\x1d\x41ntiKleptoHostNonceCommitment\x12\x12\n\ncommitment\x18\x01 \x01(\x0c\"0\n\x1a\x41ntiKleptoSignerCommitment\x12\x12\n\ncommitment\x18\x01 \x01(\x0c\"0\n\x1a\x41ntiKleptoSignatureRequest\x12\x12\n\nhost_nonce\x18\x01 \x01(\x0c\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'antiklepto_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _ANTIKLEPTOHOSTNONCECOMMITMENT._serialized_start=42
  _ANTIKLEPTOHOSTNONCECOMMITMENT._serialized_end=93
  _ANTIKLEPTOSIGNERCOMMITMENT._serialized_start=95
  _ANTIKLEPTOSIGNERCOMMITMENT._serialized_end=143
  _ANTIKLEPTOSIGNATUREREQUEST._serialized_start=145
  _ANTIKLEPTOSIGNATUREREQUEST._serialized_end=193
# @@protoc_insertion_point(module_scope)
